<?php

namespace Drupal\datafield\Plugin\Field\FieldWidget;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Element;

/**
 * Plugin implementation of the 'data_field' widget.
 *
 * @FieldWidget(
 *   id = "data_field_table_widget",
 *   label = @Translation("Data Field Table"),
 *   field_types = {"data_field"}
 * )
 */
class DataFieldTable extends DataField {

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state): array {
    $storage = $this->getFormElement($items, $delta, $element, $form, $form_state);
    if (empty($storage)) {
      return [];
    }
    $cardinality = $this->fieldDefinition->getFieldStorageDefinition()->getCardinality();
    $widget = [
      '#type' => 'table',
    ];
    if ($cardinality != -1) {
      $widget += [
        '#header' => $this->buildTableHeader(),
        '#title' => $element["#title"],
        '#caption' => $element["#title"],
        '#attributes' => [
          'class' => ['caption-top'],
        ],
      ];
    }
    return $element + $widget + $storage;
  }

  /**
   * {@inheritDoc}
   */
  private function buildTableHeader() {
    $header = [];
    $subfields = $this->getFieldSetting('columns');
    $widgetSettings = $this->getSetting('widget_settings');
    if (!empty($widgetSettings)) {
      uasort($widgetSettings, [
        'Drupal\Component\Utility\SortArray',
        'sortByWeightElement',
      ]);
      $subfields = $widgetSettings;
    }
    foreach (array_keys($subfields) as $subfield) {
      $item = $this->getFieldSetting('columns')[$subfield];
      if (!empty($widgetSettings[$subfield]) && empty($widgetSettings[$subfield]['field_display'])) {
        continue;
      }
      if (!empty($widgetSettings[$subfield]) && $widgetSettings[$subfield]['label_display'] == 'hidden') {
        $header[] = '';
        continue;
      }
      $header[] = $fieldSettings[$subfield]['label'] ?? $item['name'];
    }
    $settings = $this->getSettings();
    if (!empty($settings['operation'])) {
      $header[] = '';
    }
    return $header;
  }

  /**
   * {@inheritDoc}
   */
  public function formMultipleElements(FieldItemListInterface $items, array &$form, FormStateInterface $form_state) {
    $elements = parent::formMultipleElements($items, $form, $form_state);

    $subFormElements = Element::children($elements);

    foreach ($subFormElements as $delta) {
      if (is_numeric($delta)) {
        $childElements = Element::children($elements[$delta]);
        $element = [];
        foreach ($childElements as $child) {
          if (is_numeric($child)) {
            $element = $elements[$delta][$child];
          }
          else {
            $element[$child] = $elements[$delta][$child];
          }
        }
        unset($elements[$delta]);
        $elements[$delta] = $element;
      }
    }
    $form_id = $form_state->getBuildInfo()['base_form_id'];
    if ($form_id != 'field_config_form') {
      $elements += [
        '#widgetDataFieldTable' => TRUE,
      ];
    }
    return $elements;
  }

}
